#include <libmbtk/config.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include "arg_t.h"

MBTK_INLINE int arg_conv(long *l, double *d, const mbtk_arg_t *src, int mult)
{
	char *end;

	switch(src->type & MBTK_ARG_MASK_BASE) {
		case MBTK_INVALID:
		case MBTK_COORD:
			return -1;
		case MBTK_BOOL:
			if (mult) *d = src->v.b;
			else *l = src->v.b;
			break;
		case MBTK_LONG:
			if (mult) *d = src->v.l;
			else *l = src->v.l;
			break;
		case MBTK_ULONG:
			if (mult) *d = src->v.ul;
			else *l = src->v.ul;
			break;
		case MBTK_DOUBLE:
			if (mult) *d = src->v.d;
			else *l = src->v.d;
			break;
		case MBTK_STRING:
			if (mult) *d = strtod(src->v.s, &end);
			else *l = strtol(src->v.s, &end, 10);
			if (*end != '\0')
				return -1;
			break;
	}
	return 0;
}

int mbtk_arg_set_int(int *dst, const mbtk_arg_t *src)
{
	long int l, tmp;
	double d;
	int mult = (src->type & MBTK_ARG_MULT);

	if (arg_conv(&l, &d, src, mult) != 0)
		return -1;

	tmp = *dst;
	if (src->type & MBTK_ARG_DELTA) tmp += l;
	else if (mult) tmp = (double)tmp * d;
	else tmp = l;

	if ((tmp > INT_MAX) || (tmp < INT_MIN))
		return -1;

	*dst = tmp;
	return 0;
}

int mbtk_arg_get_int(mbtk_arg_t *dst, int src)
{
	dst->type = MBTK_LONG;
	dst->v.l = src;
	return 0;
}


int mbtk_arg_set_short(short int *dst, const mbtk_arg_t *src)
{
	long int l, tmp;
	double d;
	int mult = (src->type & MBTK_ARG_MULT);

	if (arg_conv(&l, &d, src, mult) != 0)
		return -1;

	tmp = *dst;
	if (src->type & MBTK_ARG_DELTA) tmp += l;
	else if (mult) tmp = (double)tmp * d;
	else tmp = l;

	if ((tmp > 32767) || (tmp < -32768))
		return -1;

	*dst = tmp;
	return 0;
}

int mbtk_arg_get_short(mbtk_arg_t *dst, int src)
{
	dst->type = MBTK_LONG;
	dst->v.l = src;
	return 0;
}

int mbtk_arg_set_uchar(unsigned char *dst, const mbtk_arg_t *src)
{
	long int l, tmp;
	double d;
	int mult = (src->type & MBTK_ARG_MULT);

	if (arg_conv(&l, &d, src, mult) != 0)
		return -1;

	tmp = *dst;
	if (src->type & MBTK_ARG_DELTA) tmp += l;
	else if (mult) tmp = (double)tmp * d;
	else tmp = l;

	if ((tmp > 255) || (tmp < 0))
		return -1;

	*dst = tmp;
	return 0;
}

int mbtk_arg_get_uchar(mbtk_arg_t *dst, unsigned char src)
{
	dst->type = MBTK_LONG;
	dst->v.l = src;
	return 0;
}

int mbtk_arg_set_schar(signed char *dst, const mbtk_arg_t *src)
{
	long int l, tmp;
	double d;
	int mult = (src->type & MBTK_ARG_MULT);

	if (arg_conv(&l, &d, src, mult) != 0)
		return -1;

	tmp = *dst;
	if (src->type & MBTK_ARG_DELTA) tmp += l;
	else if (mult) tmp = (double)tmp * d;
	else tmp = l;

	if ((tmp > 127) || (tmp < -128))
		return -1;

	*dst = tmp;
	return 0;
}

int mbtk_arg_get_schar(mbtk_arg_t *dst, signed char src)
{
	dst->type = MBTK_LONG;
	dst->v.l = src;
	return 0;
}

int mbtk_arg_set_color(mbtk_color_t *dst, const mbtk_arg_t *src)
{
	unsigned long int l;

	switch(src->type & MBTK_ARG_MASK_BASE) {
		case MBTK_INVALID:
		case MBTK_BOOL:
		case MBTK_DOUBLE:
		case MBTK_COORD:
			return -1;
		case MBTK_LONG:
			l = src->v.l;
			break;
		case MBTK_ULONG:
			l = src->v.ul;
			break;
		case MBTK_STRING:
			if (*src->v.s == '#') {
				char *end;
				l = strtoul(src->v.s+1, &end, 16);
				if (*end != '\0')
					return -1;
				/* in case of RGB, automatically add alpha as 0xff */
				if ((end-src->v.s) <= 7)
					l = (l << 8) | 0xff;
			}
			else
				return -1;
	}

	dst->r = (l & 0xff000000UL) >> 24;
	dst->g = (l & 0x00ff0000UL) >> 16;
	dst->b = (l & 0x0000ff00UL) >> 8;
	dst->a = (l & 0x000000ffUL);
	return 0;
}


int mbtk_arg_get_color(mbtk_arg_t *dst, mbtk_color_t src)
{
	dst->type = MBTK_ULONG;
	dst->v.l = (((unsigned long)src.r) << 24) | (((unsigned long)src.g) << 16) | (((unsigned long)src.b) << 8) | ((unsigned long)src.a);
	return 0;
}


int mbtk_arg_set_bool(mbtk_bool_t *dst, const mbtk_arg_t *src)
{
	switch(src->type & MBTK_ARG_MASK_BASE) {
		case MBTK_INVALID:
		case MBTK_COORD:
			return -1;
		case MBTK_BOOL:
			*dst = src->v.b;
			break;
		case MBTK_LONG:
			*dst = src->v.l;
			break;
		case MBTK_ULONG:
			*dst = src->v.ul;
			break;
		case MBTK_DOUBLE:
			*dst = src->v.d;
		case MBTK_STRING:
			if (
				   ((src->v.s[0] == '1') && (src->v.s[1] == '\0'))
				|| (strcmp(src->v.s, "on") == 0)
				|| (strcmp(src->v.s, "true") == 0)
				) *dst = 1;
			else if (
				   ((src->v.s[0] == '0') && (src->v.s[1] == '\0'))
				|| (strcmp(src->v.s, "off") == 0)
				|| (strcmp(src->v.s, "false") == 0)
				) *dst = 0;
			else
				return -1;
	}

	return 0;
}

int mbtk_arg_get_bool(mbtk_arg_t *dst, int src)
{
	dst->type = MBTK_LONG;
	dst->v.l = src;
	return 0;
}

/*** Combined get/set ***/

int mbtk_arg_getset_int(int *nat, mbtk_arg_t *get, const mbtk_arg_t *set)
{
	if ((get != NULL) && (mbtk_arg_get_int(get, *nat) != 0)) return -1;
	if ((set != NULL) && (mbtk_arg_set_int(nat, set) != 0)) return -1;
	return 0;
}

int mbtk_arg_getset_short(short int *nat, mbtk_arg_t *get, const mbtk_arg_t *set)
{
	if ((get != NULL) && (mbtk_arg_get_short(get, *nat) != 0)) return -1;
	if ((set != NULL) && (mbtk_arg_set_short(nat, set) != 0)) return -1;
	return 0;
}

int mbtk_arg_getset_uchar(unsigned char *nat, mbtk_arg_t *get, const mbtk_arg_t *set)
{
	if ((get != NULL) && (mbtk_arg_get_uchar(get, *nat) != 0)) return -1;
	if ((set != NULL) && (mbtk_arg_set_uchar(nat, set) != 0)) return -1;
	return 0;
}

int mbtk_arg_getset_schar(signed char *nat, mbtk_arg_t *get, const mbtk_arg_t *set)
{
	if ((get != NULL) && (mbtk_arg_get_schar(get, *nat) != 0)) return -1;
	if ((set != NULL) && (mbtk_arg_set_schar(nat, set) != 0)) return -1;
	return 0;
}

int mbtk_arg_getset_color(mbtk_color_t *nat, mbtk_arg_t *get, const mbtk_arg_t *set)
{
	if ((get != NULL) && (mbtk_arg_get_color(get, *nat) != 0)) return -1;
	if ((set != NULL) && (mbtk_arg_set_color(nat, set) != 0)) return -1;
	return 0;
}

int mbtk_arg_getset_bool(mbtk_bool_t *nat, mbtk_arg_t *get, const mbtk_arg_t *set)
{
	if ((get != NULL) && (mbtk_arg_get_bool(get, *nat) != 0)) return -1;
	if ((set != NULL) && (mbtk_arg_set_bool(nat, set) != 0)) return -1;
	return 0;
}



const char *mbtk_arg_print(char *out, int outlen, const mbtk_arg_t *a)
{
	switch(a->type & MBTK_ARG_MASK_BASE) {
		case MBTK_INVALID:
			return NULL;
		case MBTK_BOOL:
			if (outlen < 8)
				return NULL;
			strcpy(out, a->v.b ? "true" : "false");
			return out;
		case MBTK_DOUBLE:
			snprintf(out, outlen, "%f", a->v.d);
			out[outlen-1] = '\0';
			return out;
		case MBTK_LONG:
			snprintf(out, outlen, "%ld", a->v.l);
			out[outlen-1] = '\0';
			return out;
		case MBTK_ULONG:
			snprintf(out, outlen, "%lu", a->v.ul);
			out[outlen-1] = '\0';
			return out;
		case MBTK_COORD:
			snprintf(out, outlen, "%d;%d", a->v.c.x, a->v.c.y);
			out[outlen-1] = '\0';
			return out;
		case MBTK_STRING:
			return a->v.s;
	}
	return NULL;
}

