#include <libmbtk/config.h>
#include <stdlib.h>
#include <string.h>

#define UALL_INLINE MBTK_INLINE
#include <libualloc/libualloc.h>
#include <libualloc/libualloc_impl.h>
#include <libualloc/slab_api.h>
#include <libualloc/slab_impl.h>
#include "../common/fdwatch.h"
#include "../common/pipes.h"

#include "../glfw.h"

#include "mbtk_glfw_priv.h"

static int mbtk_glfw_be_init(mbtk_backend_t *be)
{
	if (!glfwInit())
		return -1;

	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);

	return 0;
}

static void mbtk_glfw_be_uninit(mbtk_backend_t *be)
{
	glfwTerminate();
}


#include "draw_low.c"
#include "draw.c"
#include "font.c"
#include "display.c"
#include "input_xlate.c"
#include "event_cbs.c"
#include "window.c"
#include "io_thread.c"
#include "cursor.c"
#include "iterate.c"
#include "clipboard.c"

int mbtk_glfw_backend(mbtk_backend_t *dst)
{
	memset(dst, 0, sizeof(mbtk_backend_t));

	dst->be_init = mbtk_glfw_be_init;
	dst->be_uninit = mbtk_glfw_be_uninit;
	dst->display_init = mbtk_glfw_display_init;
	dst->display_uninit = mbtk_glfw_display_uninit;
	dst->clipboard_set = mbtk_glfw_clipboard_set;
	dst->clipboard_get = mbtk_glfw_clipboard_get;
	dst->clipboard_free = mbtk_glfw_clipboard_free;
	dst->iterate = mbtk_glfw_iterate;
	dst->get_time_el = mbtk_glfw_get_time_el;

	dst->window_new = mbtk_glfw_window_new;
	dst->window_close = mbtk_glfw_window_close;
	dst->window_set_title = mbtk_glfw_window_set_title;
	dst->window_raise = mbtk_glfw_window_raise;
	dst->window_hide = mbtk_glfw_window_hide;
	dst->window_show = mbtk_glfw_window_show;
	dst->window_transient4 = mbtk_glfw_window_transient4;
	dst->window_set_min_size = mbtk_glfw_window_set_min_size;
	dst->window_set_size = mbtk_glfw_window_set_size;
	dst->window_get_size = mbtk_glfw_window_get_size;

	dst->draw_color = mbtk_glfw_draw_color;
	dst->draw_fillrect = mbtk_glfw_draw_fillrect;
	dst->draw_line = mbtk_glfw_draw_line;
	dst->draw_hline = mbtk_glfw_draw_hline;
	dst->draw_vline = mbtk_glfw_draw_vline;
	dst->draw_flush = mbtk_glfw_draw_flush;
	dst->draw_start = mbtk_glfw_draw_start;

	dst->resolve_font = mbtk_glfw_resolve_font;
	dst->draw_set_font = mbtk_glfw_draw_set_font;
	dst->draw_query_text = mbtk_glfw_draw_query_text;
	dst->draw_text = mbtk_glfw_draw_text;
	dst->draw_free_text = mbtk_glfw_draw_free_text;

	dst->draw_import_pixmap = mbtk_glfw_draw_import_pixmap;
	dst->draw_free_pixmap = mbtk_glfw_draw_free_pixmap;
	dst->draw_bind_pixmap = mbtk_glfw_draw_bind_pixmap;
	dst->draw_unbind_pixmap = mbtk_glfw_draw_unbind_pixmap;
	dst->draw_pixmap = mbtk_glfw_draw_pixmap;


	dst->mocu_create_named = mbtk_glfw_mocu_create_named;
	dst->mocu_create_pixmap = mbtk_glfw_mocu_create_pixmap;
	dst->mocu_free_cursor = mbtk_glfw_mocu_free_cursor;
	dst->mocu_set_cursor = mbtk_glfw_mocu_set_cursor;

	dst->watch_stdin_read = mbtk_glfw_watch_stdin_read;
	dst->watch_fd = mbtk_glfw_watch_fd;

	dst->has_alpha = 1;

	return 0;
}
