#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/widget/box.h>

static int mbtk_box_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	mbtk_box_t *box = (mbtk_box_t *)w;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = win->display->style;
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);

	if (box->framed) {
		long l, r, t, b;
		style->min_box_frame_size(w->win, sp, w->flags, &l, &r, &t, &b);
		w->hvbox.fr_l = t; w->hvbox.fr_r = b;
		w->hvbox.fr_t = t; w->hvbox.fr_b = b;
		*mx = l+r; *my = t+b;
	}
	else {
		*mx = *my = 0;
	}

	return 0;
}

static void mbtk_box_fallback_draw(mbtk_widget_t *w)
{
	mbtk_box_t *box = (mbtk_box_t *)w;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = win->display->style;
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);

	style->draw_bg_rect(win, sp, w->flags, w->hvbox.x, w->hvbox.y, w->hvbox.w, w->hvbox.h);
	if (box->framed)
		style->draw_box_frame(win, sp, w->flags, w->hvbox.x, w->hvbox.y, w->hvbox.w, w->hvbox.h);
}

static void mbtk_box_fallback_draw_overlay(mbtk_widget_t *w)
{
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = win->display->style;
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);

	style->draw_box_overlay(win, sp, w->flags, w->hvbox.x, w->hvbox.y, w->hvbox.w, w->hvbox.h);
}



static const mbtk_widget_imp_t mbtk_box_imp = {
	NULL, /* mbtk_box_destroy */

	mbtk_box_fallback_minsize,
	mbtk_box_fallback_draw,
	mbtk_box_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	NULL, /* mbtk_box_pre_insert */
	NULL  /* mbtk_box_post_insert */
};

mbtk_box_t *mbtk_box_new(mbtk_box_t *caller_alloced, int horiz)
{
	mbtk_box_t *box;
	mbtk_kw_t kw_hbox, kw_vbox;

	mbtk_kw_cache(kw_hbox, "hbox");
	mbtk_kw_cache(kw_vbox, "vbox");

	mbtk_create_wt(box, (horiz ? kw_hbox : kw_vbox), &mbtk_box_imp, caller_alloced);
	box->w.hvbox.type = (horiz ? HVBOX_HBOX : HVBOX_VBOX);

	return box;
}

mbtk_box_t *mbtk_hbox_new(mbtk_box_t *caller_alloced) { return mbtk_box_new(caller_alloced, 1); }
mbtk_box_t *mbtk_vbox_new(mbtk_box_t *caller_alloced) { return mbtk_box_new(caller_alloced, 0); }


static mbtk_widget_t *mbtk_hvbox_install(mbtk_widget_t *w, mbtk_widget_t *parent, mbtk_widget_t *at, int ins_before)
{
	hvbox_t *res = NULL;

	if (at != NULL) {
		if (ins_before)
			res = hvbox_insert_existing_box_at(&at->hvbox, &w->hvbox);
		else
			res = hvbox_append_existing_box_at(&at->hvbox, &w->hvbox);
	}
	else if (parent != NULL) {
		if (ins_before)
			res = hvbox_insert_existing_box(&parent->hvbox, &w->hvbox);
		else
			res = hvbox_append_existing_box(&parent->hvbox, &w->hvbox);
	}

	return (mbtk_widget_t *)res;
}


static mbtk_widget_t *mbtk_box_add_widget_at_(mbtk_box_t *parent, mbtk_widget_t *at, mbtk_widget_t *w, int ins_before)
{
	/* sanity checks */
	if (parent == NULL)
		return NULL;
	if ((parent->w.hvbox.type != HVBOX_HBOX) && (parent->w.hvbox.type != HVBOX_VBOX))
		return NULL;
	if (w->win != NULL)
		return NULL;

	mbtk_widget_embedded(&parent->w, w);

	mbtk_hvbox_install(w, &parent->w, at, ins_before);

	mbtk_widget_post_insert(w);

	return w;
}

mbtk_widget_t *mbtk_box_add_widget(mbtk_box_t *parent, mbtk_widget_t *w, int prepend)
{
	return mbtk_box_add_widget_at_(parent, NULL, w, prepend);
}

void mbtk_box_set_framed(mbtk_box_t *box, mbtk_bool_t has_frame)
{
	if (box->framed != has_frame) {
		box->framed = has_frame;
		mbtk_widget_changed(&box->w, 0); /* recalculate minimum size because the frame affects it */
	}
}

